using System;
using System.Web;
using System.Web.UI;

namespace Wilson.TemplatesDesign
{
	// Base Page Class cannot be Abstract for Designer.
	// Also must be Compiled to work in the Designer.
	public class BasePage : System.Web.UI.Page
	{
		private bool firstControl = true;
		private string pageTitle = "Default Title";

		// Set PageTitle in Constructor of Each Page.
		public string PageTitle {
			get { return pageTitle; }
			set { pageTitle = value; }
		}
		
		// This is the Page Template actually at Work.
		// This is Trivial Header & Footer for the Demo.
		// Probably should add Meta-Tags & StyleSheet.
		// You could also add Menus and Common Content.
		protected override void OnInit(System.EventArgs e) {
			this.Controls.AddAt(0, LoadControl("Header.ascx"));
			base.OnInit(e);
			this.Controls.Add(LoadControl("Footer.ascx"));
		}

		// This allows you to leave Header & Footer in ASPX File.
		// It will cause First and Last Static Html to be Skipped.
		// But only if obvious Html Header or Footer to be Safe.
		// This method is already used by ASP.NET for Parsing.
		// So it adds Negligible Overhead to Regular Processing.
		protected override void AddParsedSubObject(object obj) {
			bool addControl = true;
			string type = obj.GetType().ToString();
			if (type == "System.Web.UI.LiteralControl"
					|| type == "System.Web.UI.ResourceBasedLiteralControl") {
				LiteralControl control = (LiteralControl) obj;
				string html = control.Text.Trim().ToUpper();
				if (firstControl) {
					if (html.StartsWith("<!DOCTYPE") || html.StartsWith("<HTML>")) {
						addControl = false;
					}
				}
				else {
					if (html.EndsWith("</HTML>")) {
						addControl = false;
					}
				}
			}
			if (firstControl) firstControl = false;
			if (addControl) base.AddParsedSubObject(obj);
		}

		public BasePage() {
			// Note: Base Class -- Do Not Instantiate
		}
	}
	
	// Base Control Class to Enable Access to Page Properties.
	public abstract class BaseControl : System.Web.UI.UserControl
	{
		public new BasePage Page {
			get { return (BasePage)base.Page; }
		}
	}
}